@@ -21,6 +21,11 @@
 #include <math.h>
 #include <string.h>
 
+#if MAEMO_VERSION_MAJOR == 5
+#include <gdk/gdkx.h>
+#include <X11/Xatom.h>
+#endif
+
 static GtkWidget *cache_description(appdata_t *appdata, cache_t *cache) {
   return html_view(appdata, cache->long_description, 
 	   cache->long_is_html?HTML_HTML:HTML_PLAIN_TEXT, TRUE, cache, NULL);
@@ -185,6 +190,8 @@ static GtkWidget *cache_overview(cache_context_t *context) {
   vbox = gtk_vbox_new(FALSE, 4);
   
   /* hbox containing the four columns of cache details */
+  GtkWidget *pbox = portrait_box_new(FALSE, 0);
+
   GtkWidget *ihbox = gtk_hbox_new(FALSE, 0);
   
   /* vbox containing leftmost column (icon and container) */
@@ -240,6 +247,10 @@ static GtkWidget *cache_overview(cache_context_t *context) {
 
   gtk_box_pack_start_defaults(GTK_BOX(ihbox), colbox);
 
+  /* pack left two columns into portrait box */
+  gtk_box_pack_start_defaults(GTK_BOX(pbox), ihbox);
+  ihbox = gtk_hbox_new(FALSE, 0);
+
   /* ----------- vbox containing all ratings ---------- */
   colbox = gtk_vbox_new(FALSE, 0);
 
@@ -322,14 +333,42 @@ static GtkWidget *cache_overview(cache_context_t *context) {
   }
 
   gtk_box_pack_start_defaults(GTK_BOX(ihbox), colbox);
+  gtk_box_pack_start_defaults(GTK_BOX(pbox), ihbox);
  
   /* ----------------------------------------------------- */
 
-  gtk_box_pack_start(GTK_BOX(vbox), ihbox, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), pbox, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), gtk_hseparator_new(), FALSE, FALSE, 0);
 
   /* ----------------------------------------------------- */
 
+  if(cache->attribute) {
+    GtkWidget *thbox = gtk_hbox_new(FALSE, 0);
+    ihbox = gtk_hbox_new(FALSE, 0);
+
+    attribute_t *att = cache->attribute;
+    while(att) {
+      if(att->inc)
+	tip = icon_get_widget(ICON_ATT, att->id+1);
+      else
+	tip = icon_get_widget_ovl(ICON_ATT, att->id+1, 0);
+
+      gtk_box_pack_start(GTK_BOX(ihbox), tip, FALSE, FALSE, 4);
+
+#ifndef USE_MAEMO
+      gtk_tooltips_set_tip(tips, tip, _(att->name), NULL);
+#endif
+
+      att = att->next;
+    }
+
+    gtk_box_pack_start(GTK_BOX(thbox), ihbox, TRUE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), thbox, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), gtk_hseparator_new(), FALSE, FALSE, 0);
+  }
+
+  /* ----------------------------------------------------- */
+
   if(cache->short_description)
     gtk_box_pack_start_defaults(GTK_BOX(vbox), 
 	html_view(appdata, cache->short_description, 
@@ -773,98 +812,8 @@ static void on_notebook_destroy(GtkWidget *widget, gpointer user_data ) {
   g_free(context);
 }
 
-#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR >= 5)
-#define CUSTOM_NOTEBOOK
-#endif
-
-static GtkWidget *notebook_new(void) {
-#ifdef CUSTOM_NOTEBOOK
-  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
-
-  GtkWidget *notebook =  gtk_notebook_new();
-
-  /* prevents user from accidentially touching the breadcrumb trail */
-  /* (looks ugly on fremantle as it isn't themed) */
-  //  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_BOTTOM);
-
-  /* solution for fremantle: we use a row of ordinary buttons instead */
-  /* of regular tabs */
-
-  /* hide the regular tabs */
-  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
-
-  gtk_box_pack_start_defaults(GTK_BOX(vbox), notebook);
-
-  /* store a reference to the notebook in the vbox */
-  g_object_set_data(G_OBJECT(vbox), "notebook", (gpointer)notebook);
-
-  /* create a hbox for the buttons */
-  GtkWidget *hbox = gtk_hbox_new(TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-  g_object_set_data(G_OBJECT(vbox), "hbox", (gpointer)hbox);
-
-  return vbox;
-#else
-  return gtk_notebook_new();
-#endif
-}
-
-#ifdef CUSTOM_NOTEBOOK
-static void on_notebook_button_clicked(GtkWidget *button, gpointer data) {
-  GtkNotebook *nb = 
-    GTK_NOTEBOOK(g_object_get_data(G_OBJECT(data), "notebook"));
-
-  gint page = (gint)g_object_get_data(G_OBJECT(button), "page");
-  gtk_notebook_set_current_page(nb, page);
-}
-#endif
-
-static void notebook_append_page(GtkWidget *notebook, 
-				 GtkWidget *page, char *label) {
-#ifdef CUSTOM_NOTEBOOK
-  GtkNotebook *nb = 
-    GTK_NOTEBOOK(g_object_get_data(G_OBJECT(notebook), "notebook"));
-
-  gint page_num = gtk_notebook_append_page(nb, page, gtk_label_new(label));
-  GtkWidget *button = NULL;
-
-  /* select button for page 0 by default */
-  if(!page_num) {
-    button = gtk_radio_button_new_with_label(NULL, label);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
-    g_object_set_data(G_OBJECT(notebook), "group_master", (gpointer)button);
-  } else {
-    GtkWidget *master = g_object_get_data(G_OBJECT(notebook), "group_master");
-    button = gtk_radio_button_new_with_label_from_widget(
-				 GTK_RADIO_BUTTON(master), label);
-  }
-
-  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(button), FALSE);
-  g_object_set_data(G_OBJECT(button), "page", (gpointer)page_num);
-
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-	   GTK_SIGNAL_FUNC(on_notebook_button_clicked), notebook);
-
-#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR == 5)
-  hildon_gtk_widget_set_theme_size(button, 
-	   (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
-#endif  
-
-  gtk_box_pack_start_defaults(
-     GTK_BOX(g_object_get_data(G_OBJECT(notebook), "hbox")),
-     button);
-
-#else
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, gtk_label_new(label));
-#endif
-}
-
 static GObject *notebook_object(GtkWidget *notebook) {
-#ifdef CUSTOM_NOTEBOOK
-  return G_OBJECT(g_object_get_data(G_OBJECT(notebook), "notebook")); 
-#else
-  return G_OBJECT(notebook);
-#endif
+  return G_OBJECT(notebook_get_gtk_notebook(notebook));
 }
 
 GtkWidget *cache_view(appdata_t *appdata, cache_t *cache) {
@@ -955,6 +904,26 @@ void cache_dialog(appdata_t *appdata, cache_t *cache) {
 
 #else
 #ifdef USE_STACKABLE_WINDOW
+
+#if !defined(__i386__)
+/* get access to zoom buttons */
+static void
+on_window_realize(GtkWidget *widget, gpointer data) {
+  if (widget->window) {
+    unsigned char value = 1;
+    Atom hildon_zoom_key_atom = 
+      gdk_x11_get_xatom_by_name("_HILDON_ZOOM_KEY_ATOM"),
+      integer_atom = gdk_x11_get_xatom_by_name("INTEGER");
+    Display *dpy = 
+      GDK_DISPLAY_XDISPLAY(gdk_drawable_get_display(widget->window));
+    Window w = GDK_WINDOW_XID(widget->window);
+
+    XChangeProperty(dpy, w, hildon_zoom_key_atom, 
+		    integer_atom, 8, PropModeReplace, &value, 1);
+  }
+}
+#endif
+
 static void on_cache_destroy (GtkWidget *widget, appdata_t *appdata) {
   appdata->cur_cache = NULL;
 
@@ -969,12 +938,27 @@ void cache_dialog(appdata_t *appdata, cache_t *cache) {
   GtkWidget *window = hildon_stackable_window_new();
   appdata->window = HILDON_WINDOW(window);
 
+#if !defined(__i386__)
+  g_signal_connect(G_OBJECT(appdata->window), "realize", 
+		   G_CALLBACK(on_window_realize), NULL);
+#endif  
+
+  g_signal_connect(G_OBJECT(appdata->window), "key_press_event",
+		   G_CALLBACK(on_window_key_press), appdata);
+
   /* store last "cur_view" in window */
   g_object_set_data(G_OBJECT(window), "cur_view", appdata->cur_view);
 
   appdata->cur_cache = cache;
   gtk_window_set_title(GTK_WINDOW(window), cache->name);
 
+  /* make sure window can control gps */
+  g_signal_connect(G_OBJECT(window), "focus-in-event", 
+		   G_CALLBACK(on_main_focus_change), appdata);
+  
+  g_signal_connect(G_OBJECT(window), "focus-out-event", 
+		   G_CALLBACK(on_main_focus_change), appdata);
+
   /* create cache visualization widget */
   appdata->cur_view = cache_view(appdata, cache);
   gtk_container_add(GTK_CONTAINER(window), appdata->cur_view);
