@@ -67,8 +67,8 @@ static store_t store[] = {
   { "compass_locked",   STORE_BOOL,   OFFSET(compass_locked) },
   { "latitude",         STORE_FLOAT,  OFFSET(home.lat) },
   { "longitude",        STORE_FLOAT,  OFFSET(home.lon) },
-  { "gps_lat",          STORE_FLOAT,  OFFSET(gps.lat) },
-  { "gps_lon",          STORE_FLOAT,  OFFSET(gps.lon) },
+  { "gps_lat",          STORE_FLOAT,  OFFSET(gps.saved.lat) },
+  { "gps_lon",          STORE_FLOAT,  OFFSET(gps.saved.lon) },
   { "search_in",        STORE_INT,    OFFSET(search) },
   { "search_days",      STORE_INT,    OFFSET(search_days) },
   { "search_str",       STORE_STRING, OFFSET(search_str) },
@@ -89,6 +89,7 @@ static store_t store[] = {
   { "map_lon",          STORE_FLOAT,  OFFSET(map.pos.lon) },
   { "map_zoom",         STORE_INT,    OFFSET(map.zoom) },
   { "map_source",       STORE_INT,    OFFSET(map.source) },
+  { "map_dpix",         STORE_BOOL,   OFFSET(map.dpix) },
 #endif
 #ifdef ESPEAK
   { "espeak/enabled",   STORE_BOOL,   OFFSET(espeak.enabled) },
@@ -239,7 +240,7 @@ void gconf_load_state(appdata_t *appdata) {
 
   /* default positions are invalid */
   appdata->home.lat = appdata->home.lon = NAN;
-  appdata->gps.lat = appdata->gps.lon = NAN;
+  appdata->gps.saved.lat = appdata->gps.saved.lon = NAN;
   appdata->geomath.lat = appdata->geomath.lon = NAN;
 
   appdata->gt.lat = appdata->gt.lon = NAN;
