@@ -51,7 +51,9 @@
 #include <locale.h>
 #include <libintl.h>
 
+#ifndef _
 #define _(String) gettext(String)
+#endif
 #define N_(String) (String)
 
 #ifdef USE_MAEMO
@@ -65,6 +67,7 @@
 #include <hildon/hildon-help.h>
 #endif
 #if MAEMO_VERSION_MAJOR >= 5
+#define MAEMO5
 #define USE_PANNABLE_AREA
 #include <hildon/hildon-pannable-area.h>
 /* panning a gtkhtml view currently doesn't work well */
@@ -85,6 +88,8 @@
 /* all include appdata_t relies on go here ... */
 #include "gpx.h"
 #include "settings.h"
+#include "gps.h"
+#include "portrait.h"
 
 #define GPXLIST_ITEM_VALID    (1<<0)
 #define GPXLIST_ITEM_DATE     (1<<1)
@@ -108,8 +113,6 @@ typedef struct {
 
   gboolean imperial;          /* display imperial units */
 
-  pos_t gps;                  /* saved last gps position */
-
   char *image_path;           /* path to cache images in */
 
   int search;                 /* "search in" items */
@@ -217,6 +220,7 @@ typedef struct {
     int source;
     int zoom;
     pos_t pos;
+    gboolean dpix;
   } map;
 #endif
 
@@ -242,10 +246,16 @@ typedef struct {
 #ifndef USE_STACKABLE_WINDOW
   GtkWidget *vbox;    
 #endif
+
+  struct {
+    gps_mask_t set;
+    struct gps_t fix;
+    pos_t saved;                  /* saved last gps position */
+  } gps;
+
 } appdata_t;
 
 /* all includes that rely on appdata_t go here ... */
-#include "gps.h"
 #include "goto.h"
 #include "gconf.h"
 #include "html.h"
@@ -261,6 +271,7 @@ typedef struct {
 #include "garmin_export.h"
 #include "geotoad.h"
 #include "about.h"
+#include "astro.h"
 
 #ifdef ENABLE_OSM_GPS_MAP
 #include "map-tool.h"
@@ -307,4 +318,16 @@ void gpxlist_set(GtkListStore *store, GtkTreeIter *iter, gpx_t *gpx);
 void gpxlist_add(appdata_t *appdata, gpx_t *new);
 void cachelist_redraw(appdata_t *appdata);
 
+/* interface to main windows gps callback */
+pos_t *gps_get_pos(appdata_t *appdata);
+float gps_get_heading(appdata_t *appdata);
+gint gps_get_satnum(appdata_t *appdata);
+gps_sat_t *gps_get_sats(appdata_t *appdata);
+float gps_get_eph(appdata_t *appdata);
+void gps_change_state(appdata_t *appdata);
+
+/* change GPS state via main focus handling */
+gboolean on_main_focus_change(GtkWidget *, GdkEventFocus *, gpointer);
+
+
 #endif // GPXVIEW_H
