@@ -780,7 +780,7 @@ balloon_cb(osm_gps_map_balloon_event_t *event, gpointer data) {
 	   event->data.click.x, event->data.click.y);
     
     /* make the main screen jump to that cache */
-    if(!event->data.click.down) {
+    if(event->data.click.down) {
       if(context->appdata->cur_cache) {
 	printf("ERROR: no current cache should be visible!\n");
       } else {
