@@ -75,7 +75,7 @@ int checkdir(char *dir) {
   if(p[0] == '/') p++;
 
   do {
-    while(*p && *p != '/') p++;
+    while(*p && (*p != '/')) p++;
     tmp = *p;
     *p = 0;
 
@@ -87,7 +87,7 @@ int checkdir(char *dir) {
 	return -1;
       }
     } else {
-      if(!filestat.st_mode & S_IFDIR) {
+      if(!(filestat.st_mode & S_IFDIR)) {
 	printf("File %s exists and is _no_ directory\n", dir);
 	*p++ = tmp;
 	return -1;
@@ -279,8 +279,8 @@ pos_t *get_pos(appdata_t *appdata) {
   if(appdata->use_gps) {
     pos = gps_get_pos(appdata);
 
-    if(!pos) pos = &appdata->gps;   /* use saved position */
-    else     appdata->gps = *pos;   /* save position */
+    if(!pos) pos = &appdata->gps.saved;   /* use saved position */
+    else     appdata->gps.saved = *pos;   /* save position */
   }
   return pos;
 }
@@ -321,10 +321,10 @@ float distance_parse(char *str, gboolean imperial) {
     else val = NAN;
 
     /* found imperial and metric requested? convert miles into kilometers */
-    if(fimp & !imperial) val *= 1.609344;
+    if(fimp & (!imperial)) val *= 1.609344;
 
     /* found metric and imperial requested? convert kilometers into miles */
-    if(!fimp & imperial) val /= 1.609344;
+    if((!fimp) & imperial) val /= 1.609344;
   }
   return val;
 }
@@ -2251,3 +2251,175 @@ int number_editor_get_value(GtkWidget *widget) {
   return number_editor_get(widget);
 #endif
 }
+
+#ifdef MAEMO5               
+gboolean is_portrait() {
+  GdkScreen *screen = gdk_screen_get_default();
+  return (gdk_screen_get_width(screen) < gdk_screen_get_height(screen));
+}
+ 
+static
+void on_orientation_changed(GdkScreen *screen, gpointer userdata) {
+  GtkWidget *vbox = GTK_WIDGET(userdata);
+  GtkWidget *hbox0 = g_object_get_data(G_OBJECT(vbox), "hbox0");
+  GtkWidget *hbox1 = g_object_get_data(G_OBJECT(vbox), "hbox1");
+
+  GSList *xbut = g_object_get_data(G_OBJECT(vbox), "xbut");
+  if(!xbut) return;
+
+  int i;
+  for(i=0;i<g_slist_length(xbut);i++) {
+    GtkWidget *but = g_slist_nth_data(xbut, i);
+    g_object_ref(G_OBJECT(but));
+
+    if(is_portrait()) {
+      /* move button from hbox0 to hbox1 */
+      gtk_container_remove(GTK_CONTAINER(hbox0), but);
+      gtk_box_pack_start_defaults(GTK_BOX(hbox1), but);
+    } else {
+      /* move button from hbox1 to hbox0 */
+      gtk_container_remove(GTK_CONTAINER(hbox1), but);
+      gtk_box_pack_start_defaults(GTK_BOX(hbox0), but);
+    }
+    g_object_unref(G_OBJECT(but));
+  }
+  
+  gtk_window_resize(GTK_WINDOW(gtk_widget_get_toplevel(vbox)), 800, 800);
+}
+#endif
+
+GtkWidget *notebook_new(void) {
+#ifdef MAEMO5               
+  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
+
+  GtkWidget *notebook =  gtk_notebook_new();
+
+  /* solution for fremantle: we use a row of ordinary buttons instead */
+  /* of regular tabs */                                                 
+
+  /* hide the regular tabs */
+  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
+
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), notebook);
+
+  /* store a reference to the notebook in the vbox */
+  g_object_set_data(G_OBJECT(vbox), "notebook", (gpointer)notebook);
+
+  /* create hboxes for the buttons (second hbox for portrait mode) */
+  GtkWidget *hbox0 = gtk_hbox_new(TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox0, FALSE, FALSE, 0);
+  g_object_set_data(G_OBJECT(vbox), "hbox0", (gpointer)hbox0);
+  GtkWidget *hbox1 = gtk_hbox_new(TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox1, FALSE, FALSE, 0);
+  g_object_set_data(G_OBJECT(vbox), "hbox1", (gpointer)hbox1);
+
+  /* this box needs to be re-arranged for portrait mode */
+  GdkScreen *screen = gdk_screen_get_default(); 
+  g_signal_connect(screen, "size-changed", G_CALLBACK(on_orientation_changed), vbox);
+
+  return vbox;
+#else         
+  return gtk_notebook_new();
+#endif                      
+}                           
+
+GtkWidget *notebook_get_gtk_notebook(GtkWidget *notebook) {
+#ifdef MAEMO5                                           
+  return GTK_WIDGET(g_object_get_data(G_OBJECT(notebook), "notebook"));
+#else                                                                  
+  return notebook;                                                     
+#endif                                                                 
+}                                                                      
+
+#ifdef MAEMO5
+static void on_notebook_button_clicked(GtkWidget *button, gpointer data) {
+  GtkNotebook *nb =                                                       
+    GTK_NOTEBOOK(g_object_get_data(G_OBJECT(data), "notebook"));          
+
+  gint page = (gint)g_object_get_data(G_OBJECT(button), "page");
+  gtk_notebook_set_current_page(nb, page);                      
+}                                                               
+#endif             
+
+void notebook_append_page(GtkWidget *notebook, 
+                          GtkWidget *page, char *label) {
+#ifdef MAEMO5                                         
+  GtkNotebook *nb =                                      
+    GTK_NOTEBOOK(g_object_get_data(G_OBJECT(notebook), "notebook"));
+
+  gint page_num = gtk_notebook_append_page(nb, page, gtk_label_new(label));
+  GtkWidget *button = NULL;                                                
+
+  /* select button for page 0 by default */
+  if(!page_num) {                          
+    button = gtk_radio_button_new_with_label(NULL, label);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
+    g_object_set_data(G_OBJECT(notebook), "group_master", (gpointer)button);
+  } else {                                                                  
+    GtkWidget *master = g_object_get_data(G_OBJECT(notebook), "group_master");
+    button = gtk_radio_button_new_with_label_from_widget(                     
+                                 GTK_RADIO_BUTTON(master), label);            
+  }                                                                           
+
+  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(button), FALSE);
+  g_object_set_data(G_OBJECT(button), "page", (gpointer)page_num);
+
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+           GTK_SIGNAL_FUNC(on_notebook_button_clicked), notebook);
+
+  hildon_gtk_widget_set_theme_size(button,           
+           (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
+
+  gtk_box_pack_start_defaults(
+	GTK_BOX(g_object_get_data(G_OBJECT(notebook),   
+	  (!is_portrait() || page_num < 3)?"hbox0":"hbox1")), button);
+
+  /* update list of "buttons that need to be moved when switching orientation" */
+  if(page_num >= 3) {
+    GSList *xbut = g_object_get_data(G_OBJECT(notebook), "xbut");
+    xbut = g_slist_append(xbut, button);
+    g_object_set_data(G_OBJECT(notebook), "xbut", xbut);
+  }
+
+#else
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, gtk_label_new(label));
+#endif                                                                         
+}                             
+
+GtkWidget *text_wrap(char *str) {
+  GtkTextBuffer *buffer = gtk_text_buffer_new(NULL);
+
+#ifndef MAEMO5
+  GtkWidget *view = gtk_text_view_new_with_buffer(buffer);
+#else
+  GtkWidget *view = hildon_text_view_new();
+  hildon_text_view_set_buffer(HILDON_TEXT_VIEW(view), buffer);
+#endif
+
+  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_WORD);
+  gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
+  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), FALSE);
+  gtk_text_view_set_justification(GTK_TEXT_VIEW(view), GTK_JUSTIFY_LEFT); 
+
+  gtk_text_buffer_set_text(buffer, str, -1);
+
+#ifdef MAEMO5
+  /* in fremantle this is really tricky and we need to inherit the */
+  /* style from the topmost window in the stack */
+  HildonWindowStack *stack = hildon_window_stack_get_default();
+  GList *list = hildon_window_stack_get_windows(stack);
+  gtk_widget_set_style(view, GTK_WIDGET(list->data)->style);
+  g_list_free(list);
+#else
+  GtkStyle *style = gtk_widget_get_style(view);
+  gtk_widget_modify_base(view, GTK_STATE_NORMAL, &style->bg[GTK_STATE_NORMAL]);
+#endif 
+
+  return view;
+}
+
+void text_set(GtkWidget *view, char *str) {
+  GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+  gtk_text_buffer_set_text(buffer, str, -1);
+}
+
