@@ -685,12 +685,16 @@ GtkWidget *cache_notes(cache_context_t *context) {
   /* if the cache has been marked found in the logs already, there's */
   /* no need/use to be able to change all this */
   if(cache->found) {
+    check_button_set_active(context->notes.loggedw, TRUE);
+    check_button_set_active(context->notes.foundw, TRUE);
     gtk_widget_set_sensitive(table, FALSE);
-    gtk_widget_set_sensitive(view, FALSE);
-  }
+
+    gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
+    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), FALSE);
+  } else
+    gtk_text_view_set_editable(GTK_TEXT_VIEW(view), TRUE);
 
   gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_WORD);
-  gtk_text_view_set_editable(GTK_TEXT_VIEW(view), TRUE);
   gtk_text_view_set_left_margin(GTK_TEXT_VIEW(view), 2 );
   gtk_text_view_set_right_margin(GTK_TEXT_VIEW(view), 2 );
 
