@@ -18,6 +18,7 @@
  */
 
 #include "config.h" 
+
 #include <stdlib.h>  // abs
 #include <string.h>
 #include <math.h>    // M_PI/cos()
@@ -26,16 +27,32 @@
 /* OSD_DIAMETER */
 /* OSD_X, OSD_Y */
 
-#ifndef USE_CAIRO
-#error "OSD control display lacks a non-cairo implementation!"
-#endif
-
 #include <cairo.h>
 
 #include "osm-gps-map.h"
 #include "converter.h"
 #include "osm-gps-map-osd-classic.h"
 
+static OsmGpsMapSource_t map_sources[] = {
+    OSM_GPS_MAP_SOURCE_OPENSTREETMAP,
+    OSM_GPS_MAP_SOURCE_OPENSTREETMAP_RENDERER,
+    OSM_GPS_MAP_SOURCE_OPENCYCLEMAP,
+    OSM_GPS_MAP_SOURCE_OSM_PUBLIC_TRANSPORT,
+    OSM_GPS_MAP_SOURCE_GOOGLE_STREET,
+    OSM_GPS_MAP_SOURCE_VIRTUAL_EARTH_STREET,
+    OSM_GPS_MAP_SOURCE_VIRTUAL_EARTH_SATELLITE,
+    OSM_GPS_MAP_SOURCE_VIRTUAL_EARTH_HYBRID
+};
+static int num_map_sources = sizeof(map_sources)/sizeof(map_sources[0]);
+
+#ifdef OSD_DOUBLEPIXEL
+#define OSD_DPIX_EXTRA 1
+#define OSD_DPIX_SKIP  (OSD_FONT_SIZE/2)
+#else
+#define OSD_DPIX_EXTRA 0
+#define OSD_DPIX_SKIP  0
+#endif
+
 //the osd controls
 typedef struct {
     /* the offscreen representation of the OSD */
@@ -87,6 +104,14 @@ typedef struct {
     } nav;
 #endif
 
+#ifdef OSD_HEARTRATE
+    struct {
+        cairo_surface_t *surface;
+        gint rate;
+        gboolean ok;
+    } hr;
+#endif
+
 #ifdef OSD_COORDINATES
     struct {
         cairo_surface_t *surface;
@@ -103,6 +128,7 @@ typedef struct {
         gint handler_id;
         gint width, height;
         gboolean rendered;
+        gint max_h;
     } source_sel;
 #endif
 
@@ -122,8 +148,8 @@ typedef struct {
 #endif
 
 #define BALLOON_BORDER               (BALLOON_CORNER_RADIUS/2)
-#define BALLOON_WIDTH                (BALLOON_AREA_WIDTH + 2 * BALLOON_BORDER)
-#define BALLOON_HEIGHT               (BALLOON_AREA_HEIGHT + 2 * BALLOON_BORDER)
+#define BALLOON_WIDTH                (priv->balloon.rect.w + 2 * BALLOON_BORDER)
+#define BALLOON_HEIGHT               (priv->balloon.rect.h + 2 * BALLOON_BORDER)
 #define BALLOON_TRANSPARENCY         0.8
 #define POINTER_HEIGHT                20
 #define POINTER_FOOT_WIDTH            20
@@ -134,8 +160,6 @@ typedef struct {
 #define BALLOON_W  (BALLOON_WIDTH + BALLOON_SHADOW)
 #define BALLOON_H  (BALLOON_HEIGHT + POINTER_HEIGHT + BALLOON_SHADOW)
 
-#define CLOSE_BUTTON_RADIUS   (BALLOON_CORNER_RADIUS)
-
 /* draw the bubble shape. this is used twice, once for the shape and once */
 /* for the shadow */
 static void 
@@ -178,7 +202,6 @@ osd_render_balloon(osm_gps_map_osd_t *osd) {
 
     if(!priv->balloon.surface)
         return;
-
     /* get zoom */
     gint zoom;
     g_object_get(OSM_GPS_MAP(osd->widget), "zoom", &zoom, NULL);
@@ -231,13 +254,12 @@ osd_render_balloon(osm_gps_map_osd_t *osd) {
     priv->balloon.orientation = orientation;
 
     /* calculate bottom/right of box */
-    int x1 = x0 + BALLOON_WIDTH, y1 = y0 + BALLOON_HEIGHT;
+    int x1 = x0 + priv->balloon.rect.w + 2*BALLOON_BORDER;
+    int y1 = y0 + priv->balloon.rect.h + 2*BALLOON_BORDER;
 
     /* save balloon screen coordinates for later use */
     priv->balloon.rect.x = x0 + BALLOON_BORDER;
     priv->balloon.rect.y = y0 + BALLOON_BORDER;
-    priv->balloon.rect.w = x1 - x0 - 2*BALLOON_BORDER;
-    priv->balloon.rect.h = y1 - y0 - 2*BALLOON_BORDER;
 
     g_assert(priv->balloon.surface);
     cairo_t *cr = cairo_create(priv->balloon.surface);
@@ -290,10 +312,14 @@ osd_render_balloon(osm_gps_map_osd_t *osd) {
     cairo_destroy(cr);
 }
 
+#define OSD_STATE_DOWN  0
+#define OSD_STATE_UP    1
+#define OSD_STATE_CHECK 2
+
 /* return true if balloon is being displayed and if */
 /* the given coordinate is within this balloon */
 static gboolean 
-osd_balloon_check(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x, gint y) 
+osd_balloon_check(osm_gps_map_osd_t *osd, gboolean click, gint state, gint x, gint y) 
 {
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
@@ -317,7 +343,7 @@ osd_balloon_check(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x,
 
         /* handle the fact that the balloon may have been created by the */
         /* button down event */
-        if(!is_in && !down && !priv->balloon.just_created) {
+        if(!is_in && state == OSD_STATE_UP && !priv->balloon.just_created) {
             /* the user actually clicked outside the balloon */
             
             /* close the balloon! */
@@ -339,7 +365,7 @@ osd_balloon_check(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x,
             event.type = OSM_GPS_MAP_BALLOON_EVENT_TYPE_CLICK;
             event.data.click.x = x - xs;
             event.data.click.y = y - ys;
-            event.data.click.down = down;
+            event.data.click.down = (state == OSD_STATE_DOWN);
             
             priv->balloon.cb(&event, priv->balloon.data);
         }
@@ -378,18 +404,40 @@ osm_gps_map_osd_draw_balloon (OsmGpsMap *map, float latitude, float longitude,
 
     osm_gps_map_osd_clear_balloon (map);
 
-    /* allocate balloon surface */
-    priv->balloon.surface = 
-        cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 
-                                   BALLOON_W+2, BALLOON_H+2);
-
     priv->balloon.lat = latitude;
     priv->balloon.lon = longitude;
     priv->balloon.cb = cb;
     priv->balloon.data = data;
     priv->balloon.just_created = TRUE;
-
     priv->balloon.orientation = -1;
+    priv->balloon.rect.w = 0;
+    priv->balloon.rect.h = 0;
+
+    /* set default size and call app callback */
+    if(!priv->balloon.rect.w || !priv->balloon.rect.h) {
+        osm_gps_map_balloon_event_t event;
+        
+        priv->balloon.rect.w = BALLOON_AREA_WIDTH;
+        priv->balloon.rect.h = BALLOON_AREA_HEIGHT;
+
+        /* create some temporary surface, so the callback can */
+        /* e.g. determine text extents etc */
+        cairo_surface_t *surface = 
+            cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 10,10);
+
+        event.type = OSM_GPS_MAP_BALLOON_EVENT_TYPE_SIZE_REQUEST;
+        event.data.draw.rect = &priv->balloon.rect;
+        event.data.draw.cr = cairo_create(surface);
+        priv->balloon.cb(&event, priv->balloon.data);
+
+        cairo_destroy(event.data.draw.cr);
+        cairo_surface_destroy(surface);
+    }
+
+    /* allocate balloon surface */
+    priv->balloon.surface = 
+        cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 
+                                   BALLOON_W+2, BALLOON_H+2);
 
     osd_render_balloon(osd);
 
@@ -526,7 +574,7 @@ osd_shape(cairo_t *cr, GdkColor *bg, GdkColor *fg) {
 #else
 static void 
 osd_shape(cairo_t *cr) {
-    cairo_set_source_rgb (cr, OSD_COLOR_BG);
+    cairo_set_source_rgba (cr, OSD_COLOR_BG);
     cairo_fill_preserve (cr);
     cairo_set_source_rgb (cr, OSD_COLOR);
     cairo_set_line_width (cr, 1);
@@ -689,19 +737,68 @@ osd_source_content(osm_gps_map_osd_t *osd, cairo_t *cr, gint offset) {
                                     CAIRO_FONT_SLANT_NORMAL,
                                     CAIRO_FONT_WEIGHT_BOLD);
             cairo_set_font_size (cr, OSD_FONT_SIZE);
-            
-            int i, step = (priv->source_sel.height - 2*OSD_TEXT_BORDER) / 
-                OSM_GPS_MAP_SOURCE_LAST;
-            for(i=OSM_GPS_MAP_SOURCE_NULL+1;i<=OSM_GPS_MAP_SOURCE_LAST;i++) {
+
+            int i, step = (priv->source_sel.height - 2*OSD_TEXT_BORDER - OSD_DPIX_SKIP) / 
+                (num_map_sources + OSD_DPIX_EXTRA);
+            for(i=0;i<num_map_sources + OSD_DPIX_EXTRA;i++) {
                 cairo_text_extents_t extents;
-                const char *src = osm_gps_map_source_get_friendly_name(i);
+                const char *src = NULL;
+#if OSD_DPIX_EXTRA > 0
+                /* draw "double pixel" check button */
+                const char *dpix = _("Double Pixel");
+                if(i == num_map_sources) {
+                    src = dpix;
+                } else
+#endif
+                src = osm_gps_map_source_get_friendly_name( map_sources[i] );
+
                 cairo_text_extents (cr, src, &extents);
                 
                 int x = offset + OSD_S_PW + OSD_TEXT_BORDER;
-                int y = offset + step * (i-1) + OSD_TEXT_BORDER;
+                int y = offset + step * i + OSD_TEXT_BORDER;
+
+#if OSD_DPIX_EXTRA > 0
+                if(i == num_map_sources) {
+                    int skip3 = OSD_DPIX_SKIP/3;
+
+                    gboolean dpix;
+                    g_object_get(osd->widget, "double-pixel", &dpix, NULL);
+
+                    cairo_set_line_width (cr, skip3);
+
+                    /* draw seperator line */
+                    cairo_move_to (cr, x, y + skip3);
+                    cairo_rel_line_to (cr, OSD_S_AREA_W - 2*OSD_TEXT_BORDER, 0);
+                    cairo_stroke(cr);
+
+                    y += OSD_DPIX_SKIP;
+
+                    /* draw check box */
+                    cairo_move_to (cr, x, y);
+                    cairo_rel_line_to (cr, OSD_FONT_SIZE, 0);
+                    cairo_rel_line_to (cr, 0, OSD_FONT_SIZE);
+                    cairo_rel_line_to (cr, -OSD_FONT_SIZE, 0);
+                    cairo_rel_line_to (cr, 0, -OSD_FONT_SIZE);
+                    cairo_stroke(cr);
+
+                    /* draw check if needed */
+                    if(dpix) {
+                        int space = OSD_FONT_SIZE/4;
+                        cairo_move_to (cr, x + space, y + space);
+                        cairo_rel_line_to (cr, OSD_FONT_SIZE - 2*space, OSD_FONT_SIZE - 2*space);
+                        cairo_stroke(cr);
+
+                        cairo_move_to (cr, x + OSD_FONT_SIZE - space, y + space);
+                        cairo_rel_line_to (cr, -(OSD_FONT_SIZE - 2*space), OSD_FONT_SIZE - 2*space);
+                        cairo_stroke(cr);
+                    }
+
+                    x += 1.5 * OSD_FONT_SIZE;
+                }
+#endif
 
                 /* draw filled rectangle if selected */
-                if(source == i) {
+                if(source == map_sources[i]) {
                     cairo_rectangle(cr, x - OSD_TEXT_BORDER/2, 
                                     y - OSD_TEXT_SKIP, 
                                     priv->source_sel.width - OSD_TEXT_BORDER, 
@@ -713,7 +810,7 @@ osd_source_content(osm_gps_map_osd_t *osd, cairo_t *cr, gint offset) {
                     GdkColor bg = osd->widget->style->bg[GTK_STATE_NORMAL];
                     gdk_cairo_set_source_color(cr, &bg);
 #else
-                    cairo_set_source_rgb (cr, OSD_COLOR_BG);
+                    cairo_set_source_rgba (cr, OSD_COLOR_BG);
 #endif
                 }
 
@@ -721,7 +818,7 @@ osd_source_content(osm_gps_map_osd_t *osd, cairo_t *cr, gint offset) {
                 cairo_show_text (cr, src);
 
                 /* restore color */
-                if(source == i) {
+                if(source == map_sources[i]) {
 #ifndef OSD_COLOR
                     GdkColor fg = osd->widget->style->fg[GTK_STATE_NORMAL];
                     gdk_cairo_set_source_color(cr, &fg);
@@ -808,19 +905,20 @@ osd_source_reallocate(osm_gps_map_osd_t *osd) {
         cairo_set_font_size (cr, OSD_FONT_SIZE);
 
         /* calculate menu size */
-        int i, max_h = 0, max_w = 0;
-        for(i=OSM_GPS_MAP_SOURCE_NULL+1;i<=OSM_GPS_MAP_SOURCE_LAST;i++) {
-            const char *src = osm_gps_map_source_get_friendly_name(i);
+        int i,  max_w = 0;
+        priv->source_sel.max_h = 0;
+        for(i=0;i<num_map_sources;i++) {
+            const char *src = osm_gps_map_source_get_friendly_name( map_sources[i] );
             cairo_text_extents (cr, src, &extents);
 
             if(extents.width > max_w) max_w = extents.width;
-            if(extents.height > max_h) max_h = extents.height;
+            if(extents.height > priv->source_sel.max_h) priv->source_sel.max_h = extents.height;
         }
         cairo_destroy(cr);
        
         priv->source_sel.width  = max_w + 2*OSD_TEXT_BORDER;
-        priv->source_sel.height = OSM_GPS_MAP_SOURCE_LAST * 
-            (max_h + 2*OSD_TEXT_SKIP) + 2*OSD_TEXT_BORDER;
+        priv->source_sel.height = (num_map_sources + OSD_DPIX_EXTRA) * 
+            (priv->source_sel.max_h + 2*OSD_TEXT_SKIP) + 2*OSD_TEXT_BORDER + OSD_DPIX_SKIP;
 
         w = OSD_S_EXP_W;
         h = OSD_S_EXP_H;
@@ -911,7 +1009,7 @@ osd_source_toggle(osm_gps_map_osd_t *osd)
 
 /* check if the user clicked inside the source selection area */
 static osd_button_t
-osd_source_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
+osd_source_check(osm_gps_map_osd_t *osd, gint state, gint x, gint y) {
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->source_sel.expanded)
@@ -929,7 +1027,7 @@ osd_source_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
         /* really within puller shape? */
         if(x > Z_RAD || osm_gps_map_in_circle(x, y, Z_RAD, Z_RAD, Z_RAD)) {
             /* expand source selector */
-            if(down)
+            if(state == OSD_STATE_DOWN)
                 osd_source_toggle(osd);
 
             /* tell upper layers that user clicked some background element */
@@ -949,26 +1047,40 @@ osd_source_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
            y > 0 &&
            y < OSD_S_EXP_H) {
             
-            int step = (priv->source_sel.height - 2*OSD_TEXT_BORDER) 
-                / OSM_GPS_MAP_SOURCE_LAST;
+            int step = (priv->source_sel.height - 2*OSD_TEXT_BORDER - OSD_DPIX_SKIP) 
+                / (num_map_sources + OSD_DPIX_EXTRA);
 
             y -= OSD_TEXT_BORDER - OSD_TEXT_SKIP;
-            y /= step;
-            y += 1;
+            int py = y / step;
 
-            if(down) {
+            if(state == OSD_STATE_DOWN) {
                 gint old = 0;
                 g_object_get(osd->widget, "map-source", &old, NULL);
 
-                if(y > OSM_GPS_MAP_SOURCE_NULL &&
-                   y <= OSM_GPS_MAP_SOURCE_LAST &&
-                   old != y) {
-                    g_object_set(osd->widget, "map-source", y, NULL);
+                if(py >= 0 &&
+                   py < num_map_sources &&
+                   old != map_sources[py]) {
+                    g_object_set(osd->widget, "map-source", map_sources[py], NULL);
                     
                     osd_render_source_sel(osd, TRUE);
                     osm_gps_map_repaint(OSM_GPS_MAP(osd->widget));
                     osm_gps_map_redraw(OSM_GPS_MAP(osd->widget));
                 }
+
+#if OSD_DPIX_EXTRA > 0
+                if(py >= num_map_sources) {
+                    y -= num_map_sources * (priv->source_sel.max_h + 2*OSD_TEXT_SKIP ) +
+                        OSD_TEXT_SKIP + OSD_DPIX_SKIP;
+                    if(y >= 0) {
+                        gboolean dpix = 0;
+                        g_object_get(osd->widget, "double-pixel", &dpix, NULL);
+                        g_object_set(osd->widget, "double-pixel", !dpix, NULL);
+                        osd_render_source_sel(osd, TRUE);
+                        osm_gps_map_repaint(OSM_GPS_MAP(osd->widget));
+                        osm_gps_map_redraw(OSM_GPS_MAP(osd->widget));
+                    }
+                }
+#endif
             }
             
             /* return "clicked in OSD background" to prevent further */
@@ -1232,8 +1344,6 @@ static float get_distance(float lat1, float lon1, float lat2, float lon2) {
   float aob = acos(cos(lat1) * cos(lat2) * cos(lon2 - lon1) +
 		   sin(lat1) * sin(lat2));
 
-  //  return(aob * 3959.0);   /* great circle radius in miles */
-
   return(aob * 6371000.0);     /* great circle radius in meters */
 }
 
@@ -1365,10 +1475,10 @@ osd_render_nav(osm_gps_map_osd_t *osd)
 
 /* check if the user clicked inside the source selection area */
 static osd_button_t
-osd_nav_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
+osd_nav_check(osm_gps_map_osd_t *osd, gint state, gint x, gint y) {
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
     
-    if(!priv->nav.surface || down) 
+    if(!priv->nav.surface || state == OSD_STATE_DOWN) 
         return OSD_NONE;
 
     x -= OSD_X;
@@ -1439,12 +1549,138 @@ osm_gps_map_osd_draw_nav (OsmGpsMap *map, gboolean imperial,
 
 #endif // OSD_NAV
 
+#ifdef OSD_HEARTRATE
+#ifndef OSD_HR_FONT_SIZE
+#define OSD_HR_FONT_SIZE 60
+#endif
+#define OSD_HR_W  3*OSD_HR_FONT_SIZE
+#define OSD_HR_H  OSD_HR_FONT_SIZE
+
+#ifndef OSD_HR_Y
+#define OSD_HR_Y OSD_Y
+#endif
+
+static void
+osd_render_heart_shape(cairo_t *cr, gint x, gint y, gint s) {
+    cairo_move_to (cr, x-s, y-s/4);
+    cairo_curve_to (cr, x-s, y-s,     x,   y-s,     x,   y-s/4);
+    cairo_curve_to (cr, x,   y-s,     x+s, y-s,     x+s, y-s/4);
+    cairo_curve_to (cr, x+s, y+s/2,   x,   y+3*s/4, x,   y+s);
+    cairo_curve_to (cr, x,   y+3*s/4, x-s, y+s/2,   x-s, y-s/4);
+}
+
+static void 
+osd_render_heart(cairo_t *cr, gint x, gint y, gint s, gboolean ok) {
+    /* xyz */
+
+    cairo_set_line_join (cr, CAIRO_LINE_JOIN_ROUND);
+
+    osd_render_heart_shape(cr, x, y, s);
+    cairo_set_source_rgb (cr, 1, 1, 1);
+    cairo_set_line_width (cr, s/2);
+    cairo_stroke(cr);
+
+    osd_render_heart_shape(cr, x, y, s);
+    if(ok) cairo_set_source_rgb (cr, 1, 0, 0);
+    else   cairo_set_source_rgb (cr, 0.8, 0.8, 0.8);
+    cairo_fill_preserve (cr);
+    cairo_set_line_width (cr, s/5);
+
+    cairo_stroke(cr);
+}
+
+static void
+osd_render_hr(osm_gps_map_osd_t *osd) 
+{
+    osd_priv_t *priv = (osd_priv_t*)osd->priv; 
+
+    if(!priv->hr.surface)
+        return;
+
+    /* first fill with transparency */
+    g_assert(priv->hr.surface);
+    cairo_t *cr = cairo_create(priv->hr.surface);
+    cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+    cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 0.0);
+    cairo_paint(cr);
+    cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+
+    cairo_select_font_face (cr, "Sans",
+                            CAIRO_FONT_SLANT_NORMAL,
+                            CAIRO_FONT_WEIGHT_BOLD);
+    cairo_set_font_size (cr, OSD_HR_FONT_SIZE);
+
+    char str[5];
+    if(priv->hr.rate > 0) 
+        snprintf(str, 4, "%03u", priv->hr.rate);
+    else if(priv->hr.rate == OSD_HR_INVALID) 
+        strcpy(str, "?");
+    else if(priv->hr.rate == OSD_HR_ERROR) 
+        strcpy(str, "Err");
+    else 
+        g_assert(priv->hr.rate != OSD_HR_NONE);
+
+    cairo_text_extents_t extents;
+    cairo_text_extents (cr, str, &extents);
+
+    cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+    cairo_set_line_width (cr, OSD_HR_FONT_SIZE/10);
+
+    cairo_move_to (cr, OSD_HR_W/2 - extents.width/2 + OSD_HR_FONT_SIZE/4, 
+                       OSD_HR_H/2 - extents.y_bearing/2);
+    cairo_text_path (cr, str);
+    cairo_stroke (cr);
+
+    cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+    cairo_move_to (cr, OSD_HR_W/2 - extents.width/2 + OSD_HR_FONT_SIZE/4, 
+                       OSD_HR_H/2 - extents.y_bearing/2);
+    cairo_show_text (cr, str);
+
+    osd_render_heart(cr, OSD_HR_W/2 - extents.width/2 - OSD_HR_FONT_SIZE/8, 
+                     OSD_HR_H/2, OSD_HR_FONT_SIZE/5, priv->hr.ok);
+    
+
+    cairo_destroy(cr);
+}
+
+void 
+osm_gps_map_osd_draw_hr (OsmGpsMap *map, gboolean ok, gint rate) {
+    g_return_if_fail (OSM_IS_GPS_MAP (map));
+
+    osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
+    g_return_if_fail (osd);
+
+    osd_priv_t *priv = (osd_priv_t*)osd->priv; 
+    g_return_if_fail (priv);
+
+    /* allocate heart rate surface */
+    if(rate != OSD_HR_NONE && !priv->hr.surface)
+        priv->hr.surface = 
+            cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 
+                                       OSD_HR_W, OSD_HR_H);
+
+    if(rate == OSD_HR_NONE && priv->hr.surface) {
+        cairo_surface_destroy(priv->hr.surface);
+        priv->hr.surface = NULL;
+    }
+
+    if(priv->hr.rate != rate || priv->hr.ok != ok) {
+        priv->hr.rate = rate;
+        priv->hr.ok = ok;
+        osd_render_hr(osd);
+    }
+    
+    osm_gps_map_redraw(map);
+}
+ 
+#endif // OSD_HEARTRATE
+ 
 static osd_button_t
-osd_check_int(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x, gint y) {
+osd_check_int(osm_gps_map_osd_t *osd, gboolean click, gint state, gint x, gint y) {
     osd_button_t but = OSD_NONE;
 
 #ifdef OSD_BALLOON
-    if(down) {
+    if(state == OSD_STATE_DOWN) {
         /* needed to handle balloons that are created at click */
         osd_priv_t *priv = (osd_priv_t*)osd->priv; 
         priv->balloon.just_created = FALSE;
@@ -1453,13 +1689,13 @@ osd_check_int(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x, gin
 
 #ifdef OSD_SOURCE_SEL
     /* the source selection area is handles internally */
-    but = osd_source_check(osd, down, x, y);
+    but = osd_source_check(osd, state, x, y);
 #endif
 
 #ifdef OSD_NAV
     if(but == OSD_NONE) {
         /* the source selection area is handles internally */
-        but = osd_nav_check(osd, down, x, y);
+        but = osd_nav_check(osd, state, x, y);
     }
 #endif
         
@@ -1488,7 +1724,7 @@ osd_check_int(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x, gin
 #ifdef OSD_BALLOON
     if(but == OSD_NONE) {
         /* check if user clicked into balloon */
-        if(osd_balloon_check(osd, click, down, x, y)) 
+        if(osd_balloon_check(osd, click, state, x, y)) 
             but = OSD_BG;
     }
 #endif
@@ -1937,6 +2173,17 @@ osd_draw(osm_gps_map_osd_t *osd, GdkDrawable *drawable)
     cairo_paint(cr);
 #endif
 
+#ifdef OSD_HEARTRATE
+    if(priv->hr.surface) {
+        x = (osd->widget->allocation.width - OSD_HR_W)/2;
+        y = OSD_HR_Y;
+        if(y < 0) y += osd->widget->allocation.height - OSD_COORDINATES_H;
+        
+        cairo_set_source_surface(cr, priv->hr.surface, x, y);
+        cairo_paint(cr);
+    }
+#endif
+
 #ifdef OSD_BALLOON
     if(priv->balloon.surface) {
  
@@ -2029,11 +2276,17 @@ osd_free(osm_gps_map_osd_t *osd)
 #endif
 
 #ifdef OSD_BALLOON
-    if (priv->balloon.surface)
+    if (priv->balloon.surface) 
          cairo_surface_destroy(priv->balloon.surface);
 #endif
 
+#ifdef OSD_HEARTRATE
+    if (priv->hr.surface) 
+         cairo_surface_destroy(priv->hr.surface);
+#endif
+
     g_free(priv);
+    osd->priv = NULL;
 }
 
 static gboolean
@@ -2049,7 +2302,7 @@ osd_busy(osm_gps_map_osd_t *osd)
 
 static osd_button_t
 osd_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
-    return osd_check_int(osd, TRUE, down, x, y);
+    return osd_check_int(osd, TRUE, down?OSD_STATE_DOWN:OSD_STATE_UP, x, y);
 }
 
 static osm_gps_map_osd_t osd_classic = {
@@ -2084,6 +2337,10 @@ osm_gps_map_osd_classic_init(OsmGpsMap *map)
     priv->balloon.lon = OSM_GPS_MAP_INVALID;
 #endif
 
+#ifdef OSD_HEARTRATE
+    priv->hr.rate = OSD_HR_NONE;
+#endif
+
     osm_gps_map_register_osd(map, &osd_classic);
 }
 
@@ -2111,5 +2368,5 @@ osm_gps_map_osd_check(OsmGpsMap *map, gint x, gint y) {
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
     g_return_val_if_fail (osd, OSD_NONE);
     
-    return osd_check_int(osd, FALSE, TRUE, x, y);
+    return osd_check_int(osd, FALSE, OSD_STATE_CHECK, x, y);
 }
